@@ -1,5 +1,8 @@
-from typing import List, Dict, Optional, Any, Union
+import mimetypes
+import os
 from enum import Enum
+from typing import Any, Dict, List, Optional, Union
+
 from pydantic import BaseModel, model_validator
 from pydantic.json import pydantic_encoder
 
@@ -12,11 +15,34 @@ class GSlidesBaseModel(BaseModel):
         return super().model_dump(exclude_none=True, mode="json")
 
 
+class Unit(Enum):
+    """Enumeration of possible units of measurement.
+
+    Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/Unit
+    """
+
+    UNIT_UNSPECIFIED = "UNIT_UNSPECIFIED"
+    """The units are unknown."""
+
+    EMU = "EMU"
+    """An English Metric Unit (EMU) is defined as 1/360,000 of a centimeter and thus there are 914,400 EMUs per inch, and 12,700 EMUs per point."""
+
+    PT = "PT"
+    """A point, 1/72 of an inch."""
+
+
+class OutputUnit(Enum):
+    """Used in PageElementBase.absolute_* methods"""
+
+    IN = "in"
+    CM = "cm"
+
+
 class Dimension(GSlidesBaseModel):
     """Represents a size dimension with magnitude and unit."""
 
     magnitude: float
-    unit: Optional[str] = None
+    unit: Unit
 
 
 class Size(GSlidesBaseModel):
@@ -36,6 +62,43 @@ class Transform(GSlidesBaseModel):
     unit: Optional[str] = None  # Make optional to preserve original JSON exactly
 
 
+class AffineTransform(GSlidesBaseModel):
+    """AffineTransform uses a 3x3 matrix with an implied last row of [ 0 0 1 ] to transform source coordinates (x,y) into destination coordinates (x', y').
+
+    The transformation follows:
+    [ x']   [  scaleX  shearX  translateX  ] [ x ]
+    [ y'] = [  shearY  scaleY  translateY  ] [ y ]
+    [ 1 ]   [      0       0         1     ] [ 1 ]
+
+    After transformation:
+    x' = scaleX * x + shearX * y + translateX;
+    y' = scaleY * y + shearY * x + translateY;
+
+    Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations.pages/other#Page.AffineTransform
+    """
+
+    scaleX: float
+    """The X coordinate scaling element."""
+
+    scaleY: float
+    """The Y coordinate scaling element."""
+
+    shearX: float
+    """The X coordinate shearing element."""
+
+    shearY: float
+    """The Y coordinate shearing element."""
+
+    translateX: float
+    """The X coordinate translation element."""
+
+    translateY: float
+    """The Y coordinate translation element."""
+
+    unit: Optional[Unit] = None
+    """The units for translate elements."""
+
+
 class RgbColor(GSlidesBaseModel):
     """Represents an RGB color."""
 
@@ -342,6 +405,73 @@ class Image(GSlidesBaseModel):
         return self
 
 
+class ImageData(BaseModel):
+    """Container for retrieved image data with metadata."""
+
+    content: bytes
+    """Raw image data as bytes."""
+
+    mime_type: str
+    """MIME type of the image (e.g., 'image/jpeg', 'image/png')."""
+
+    filename: Optional[str] = None
+    """Optional filename hint for saving."""
+
+    @classmethod
+    def from_file(cls, path: str):
+        """Read image data from a file."""
+        with open(path, "rb") as f:
+            content = f.read()
+        mime_type = mimetypes.guess_type(path)[0]
+        return cls(content=content, mime_type=mime_type, filename=os.path.abspath(path))
+
+    def save_to_file(self, path: str) -> str:
+        """Save image data to a file.
+
+        Args:
+            path: File path to save to. If it's a directory, uses filename hint.
+
+        Returns:
+            str: The actual path where the file was saved.
+
+        Raises:
+            ValueError: If path is a directory but no filename hint is available.
+            OSError: If file cannot be written.
+        """
+        if os.path.isdir(path):
+            if not self.filename:
+                # Generate filename from MIME type
+                ext = mimetypes.guess_extension(self.mime_type) or ".bin"
+                filename = f"image{ext}"
+            else:
+                filename = self.filename
+            file_path = os.path.join(path, filename)
+        else:
+            file_path = path
+
+        # Ensure parent directory exists
+        parent_dir = os.path.dirname(file_path)
+        if parent_dir:
+            try:
+                os.makedirs(parent_dir, exist_ok=True)
+            except OSError as e:
+                raise OSError(f"Cannot create directory {parent_dir}: {e}") from e
+
+        # Write the file
+        try:
+            with open(file_path, "wb") as f:
+                f.write(self.content)
+        except OSError as e:
+            raise OSError(f"Cannot write file {file_path}: {e}") from e
+
+        return file_path
+
+    def get_extension(self) -> str:
+        """Get file extension based on MIME type."""
+        ext = mimetypes.guess_extension(self.mime_type)
+        return ext or ".bin"
+
+
 class VideoSourceType(Enum):
     """Enumeration of possible video source types."""
 
@@ -437,7 +567,7 @@ class SpeakerSpotlight(GSlidesBaseModel):
 class Group(GSlidesBaseModel):
     """Represents a group of page elements."""
 
-    children: Optional[List[Any]] = None  # This will be a list of PageElement objects
+    children: List["PageElement"]  # This will be a list of PageElement objects
 
 
 class PropertyState(Enum):
@@ -621,3 +751,13 @@ class ThumbnailProperties(GSlidesBaseModel):
 
     If you don't specify the size, the server chooses a default size for the image.
     """
+
+
+# https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#pageelementproperties
+# Note: When you initially create a PageElement, the API may modify the values of both size and transform, but the visual size will be unchanged.
+class PageElementProperties(GSlidesBaseModel):
+    """Represents properties of a page element."""
+
+    pageObjectId: Optional[str] = None
+    size: Optional[Size] = None
+    transform: Optional[Transform] = None
