@@ -1,37 +1,26 @@
-from typing import Dict, Any, List, Union, Annotated, Optional
+import logging
+import mimetypes
+import uuid
+from typing import Annotated, Any, Dict, List, Optional, Union
+from urllib.parse import urlparse
 
-from pydantic import Field, Discriminator, Tag, field_validator
+import requests
+from pydantic import Discriminator, Field, Tag, field_validator
 
-
-from gslides_api.domain import (
-    Table,
-    Image,
-    Video,
-    Line,
-    WordArt,
-    SheetsChart,
-    SpeakerSpotlight,
-    Group,
-    ImageReplaceMethod,
-)
-from gslides_api.element.base import PageElementBase, ElementKind
-from gslides_api.client import api_client, GoogleAPIClient
+from gslides_api.client import GoogleAPIClient, api_client
+from gslides_api.domain import (Group, Image, ImageData, ImageReplaceMethod, Line,
+                                SheetsChart, SpeakerSpotlight, Table, Video,
+                                WordArt)
+from gslides_api.element.base import ElementKind, PageElementBase
 from gslides_api.element.shape import ShapeElement
-from gslides_api.utils import dict_to_dot_separated_field_list, image_url_is_valid
-from gslides_api.request.request import (
-    UpdateImagePropertiesRequest,
-    ReplaceImageRequest,
-    GSlidesAPIRequest,
-    UpdateVideoPropertiesRequest,
-    UpdateLinePropertiesRequest,
-    UpdateSheetsChartPropertiesRequest,
-    CreateImageRequest,
-    CreateVideoRequest,
-    CreateLineRequest,
-    CreateWordArtRequest,
-    CreateSheetsChartRequest,
-)
+from gslides_api.request.request import (  # UpdateSheetsChartPropertiesRequest,; CreateWordArtRequest,
+    CreateImageRequest, CreateLineRequest, CreateSheetsChartRequest,
+    CreateVideoRequest, GSlidesAPIRequest, ReplaceImageRequest,
+    UpdateImagePropertiesRequest, UpdateLinePropertiesRequest,
+    UpdateVideoPropertiesRequest)
 from gslides_api.request.table import CreateTableRequest
+from gslides_api.utils import (dict_to_dot_separated_field_list,
+                               image_url_is_valid)
 
 
 def element_discriminator(v: Any) -> str:
@@ -122,6 +111,49 @@ class ImageElement(PageElementBase):
     def validate_type(cls, v):
         return ElementKind.IMAGE
 
+    @staticmethod
+    def create_image_request_like(
+        e: PageElementBase,
+        image_id: str | None = None,
+        url: str | None = None,
+        parent_id: str | None = None,
+    ) -> List[GSlidesAPIRequest]:
+        """Create a request to create an image element like the given element."""
+        url = (
+            url
+            or "https://upload.wikimedia.org/wikipedia/commons/2/2d/Logo_Google_blanco.png"
+        )
+        element_properties = e.element_properties(parent_id or e.slide_id)
+        request = CreateImageRequest(
+            objectId=image_id,
+            elementProperties=element_properties,
+            url=url,
+        )
+        return [request]
+
+    @staticmethod
+    def create_image_element_like(
+        e: PageElementBase,
+        api_client: GoogleAPIClient | None = None,
+        parent_id: str | None = None,
+        url: str | None = None,
+    ) -> str:
+        from gslides_api.page.slide import Slide
+
+        api_client = api_client or globals()["api_client"]
+        parent_id = parent_id or e.slide_id
+
+        # Create the image element
+        image_id = uuid.uuid4().hex
+        requests = ImageElement.create_image_request_like(
+            e,
+            parent_id=parent_id,
+            url=url,
+            image_id=image_id,
+        )
+        api_client.batch_update(requests, e.presentation_id)
+        return image_id
+
     def create_request(self, parent_id: str) -> List[GSlidesAPIRequest]:
         """Convert an ImageElement to a create request for the Google Slides API."""
         element_properties = self.element_properties(parent_id)
@@ -135,7 +167,10 @@ class ImageElement(PageElementBase):
         """Convert an ImageElement to an update request for the Google Slides API."""
         requests = self.alt_text_update_request(element_id)
 
-        if hasattr(self.image, "imageProperties") and self.image.imageProperties is not None:
+        if (
+            hasattr(self.image, "imageProperties")
+            and self.image.imageProperties is not None
+        ):
             image_properties = self.image.imageProperties.to_api_format()
             # "fields": "*" causes an error
             request = UpdateImagePropertiesRequest(
@@ -154,7 +189,10 @@ class ImageElement(PageElementBase):
         description = self.title or "Image"
         return f"![{description}]({url})"
 
-    def _replace_image_requests(self, new_url: str, method: ImageReplaceMethod | None = None):
+    @staticmethod
+    def _replace_image_requests(
+        objectId: str, new_url: str, method: ImageReplaceMethod | None = None
+    ):
         """
         Replace image by URL with validation.
 
@@ -173,7 +211,7 @@ class ImageElement(PageElementBase):
             raise ValueError(f"Image URL is not accessible or invalid: {new_url}")
 
         request = ReplaceImageRequest(
-            imageObjectId=self.objectId,
+            imageObjectId=objectId,
             url=new_url,
             imageReplaceMethod=method.value if method is not None else None,
         )
@@ -185,18 +223,123 @@ class ImageElement(PageElementBase):
         file: str | None = None,
         method: ImageReplaceMethod | None = None,
         api_client: Optional[GoogleAPIClient] = None,
+    ):
+        # if url is None and file is None:
+        #     raise ValueError("Must specify either url or file")
+        # if url is not None and file is not None:
+        #     raise ValueError("Must specify either url or file, not both")
+        #
+        # client = api_client or globals()["api_client"]
+        # if file is not None:
+        #     url = client.upload_image_to_drive(file)
+        #
+        # requests = self._replace_image_requests(url, method)
+        # return client.batch_update(requests, self.presentation_id)
+        return ImageElement.replace_image_from_id(
+            self.objectId,
+            self.presentation_id,
+            url=url,
+            file=file,
+            method=method,
+            api_client=api_client,
+        )
+
+    @staticmethod
+    def replace_image_from_id(
+        image_id: str,
+        presentation_id: str,
+        url: str | None = None,
+        file: str | None = None,
+        method: ImageReplaceMethod | None = None,
+        api_client: Optional[GoogleAPIClient] = None,
     ):
         if url is None and file is None:
             raise ValueError("Must specify either url or file")
         if url is not None and file is not None:
             raise ValueError("Must specify either url or file, not both")
 
-        client = api_client or globals()['api_client']
+        client = api_client or globals()["api_client"]
         if file is not None:
             url = client.upload_image_to_drive(file)
 
-        requests = self._replace_image_requests(url, method)
-        return client.batch_update(requests, self.presentation_id)
+        requests = ImageElement._replace_image_requests(image_id, url, method)
+        return client.batch_update(requests, presentation_id)
+
+    def get_image_data(self) -> ImageData:
+        """Retrieve the actual image data from Google Slides.
+        
+        Returns:
+            ImageData: Container with image bytes, MIME type, and optional filename.
+            
+        Raises:
+            ValueError: If no image URL is available.
+            requests.RequestException: If the image download fails.
+        """
+        logger = logging.getLogger(__name__)
+        
+        # Prefer contentUrl over sourceUrl as it's Google's cached version
+        url = self.image.contentUrl or self.image.sourceUrl
+        
+        if not url:
+            logger.error("No image URL available for element %s", self.objectId)
+            raise ValueError("No image URL available (neither contentUrl nor sourceUrl)")
+        
+        logger.info("Downloading image from URL: %s", url)
+        
+        try:
+            # Download the image with retries for common network issues
+            response = requests.get(url, timeout=30)
+            response.raise_for_status()
+            
+            content_length = len(response.content)
+            logger.debug("Downloaded %d bytes from %s", content_length, url)
+            
+            if content_length == 0:
+                logger.warning("Downloaded empty image content from %s", url)
+                raise ValueError("Downloaded image is empty")
+                
+        except requests.exceptions.Timeout as e:
+            logger.error("Timeout downloading image from %s: %s", url, e)
+            raise requests.RequestException(f"Timeout downloading image: {e}") from e
+        except requests.exceptions.RequestException as e:
+            logger.error("Failed to download image from %s: %s", url, e)
+            raise
+        
+        # Determine MIME type
+        mime_type = response.headers.get('content-type', 'application/octet-stream')
+        logger.debug("Content-Type header: %s", mime_type)
+        
+        # If MIME type is not image-specific, try to guess from URL
+        if not mime_type.startswith('image/'):
+            parsed_url = urlparse(url)
+            path = parsed_url.path
+            if path:
+                guessed_type, _ = mimetypes.guess_type(path)
+                if guessed_type and guessed_type.startswith('image/'):
+                    logger.debug("Guessed MIME type from URL: %s -> %s", path, guessed_type)
+                    mime_type = guessed_type
+                else:
+                    logger.warning("Could not determine image MIME type, using default: %s", mime_type)
+        
+        # Extract filename from URL if possible
+        filename = None
+        parsed_url = urlparse(url)
+        if parsed_url.path:
+            filename = parsed_url.path.split('/')[-1]
+            # Only keep if it looks like a filename with extension
+            if '.' not in filename:
+                filename = None
+            else:
+                logger.debug("Extracted filename from URL: %s", filename)
+        
+        logger.info("Successfully retrieved image: %d bytes, MIME type: %s", 
+                   content_length, mime_type)
+        
+        return ImageData(
+            content=response.content,
+            mime_type=mime_type,
+            filename=filename
+        )
 
 
 class VideoElement(PageElementBase):
@@ -230,7 +373,10 @@ class VideoElement(PageElementBase):
         """Convert a VideoElement to an update request for the Google Slides API."""
         requests = self.alt_text_update_request(element_id)
 
-        if hasattr(self.video, "videoProperties") and self.video.videoProperties is not None:
+        if (
+            hasattr(self.video, "videoProperties")
+            and self.video.videoProperties is not None
+        ):
             video_properties = self.video.videoProperties.to_api_format()
             video_request = UpdateVideoPropertiesRequest(
                 objectId=element_id,
@@ -268,7 +414,10 @@ class LineElement(PageElementBase):
         """Convert a LineElement to an update request for the Google Slides API."""
         requests = self.alt_text_update_request(element_id)
 
-        if hasattr(self.line, "lineProperties") and self.line.lineProperties is not None:
+        if (
+            hasattr(self.line, "lineProperties")
+            and self.line.lineProperties is not None
+        ):
             line_properties = self.line.lineProperties.to_api_format()
             line_request = UpdateLinePropertiesRequest(
                 objectId=element_id,
@@ -285,7 +434,9 @@ class WordArtElement(PageElementBase):
 
     wordArt: WordArt
     type: ElementKind = Field(
-        default=ElementKind.WORD_ART, description="The type of page element", exclude=True
+        default=ElementKind.WORD_ART,
+        description="The type of page element",
+        exclude=True,
     )
 
     @field_validator("type")
@@ -293,18 +444,18 @@ class WordArtElement(PageElementBase):
     def validate_type(cls, v):
         return ElementKind.WORD_ART
 
-    def create_request(self, parent_id: str) -> List[GSlidesAPIRequest]:
-        """Convert a WordArtElement to a create request for the Google Slides API."""
-        element_properties = self.element_properties(parent_id)
-
-        if not self.wordArt.renderedText:
-            raise ValueError("Rendered text is required for Word Art")
-
-        request = CreateWordArtRequest(
-            elementProperties=element_properties,
-            renderedText=self.wordArt.renderedText,
-        )
-        return [request]
+    # def create_request(self, parent_id: str) -> List[GSlidesAPIRequest]:
+    #     """Convert a WordArtElement to a create request for the Google Slides API."""
+    #     element_properties = self.element_properties(parent_id)
+    #
+    #     if not self.wordArt.renderedText:
+    #         raise ValueError("Rendered text is required for Word Art")
+    #
+    #     request = CreateWordArtRequest(
+    #         elementProperties=element_properties,
+    #         renderedText=self.wordArt.renderedText,
+    #     )
+    #     return [request]
 
     def element_to_update_request(self, element_id: str) -> List[GSlidesAPIRequest]:
         """Convert a WordArtElement to an update request for the Google Slides API."""
@@ -317,7 +468,9 @@ class SheetsChartElement(PageElementBase):
 
     sheetsChart: SheetsChart
     type: ElementKind = Field(
-        default=ElementKind.SHEETS_CHART, description="The type of page element", exclude=True
+        default=ElementKind.SHEETS_CHART,
+        description="The type of page element",
+        exclude=True,
     )
 
     @field_validator("type")
@@ -330,7 +483,9 @@ class SheetsChartElement(PageElementBase):
         element_properties = self.element_properties(parent_id)
 
         if not self.sheetsChart.spreadsheetId or not self.sheetsChart.chartId:
-            raise ValueError("Spreadsheet ID and Chart ID are required for Sheets Chart")
+            raise ValueError(
+                "Spreadsheet ID and Chart ID are required for Sheets Chart"
+            )
 
         request = CreateSheetsChartRequest(
             elementProperties=element_properties,
@@ -339,23 +494,23 @@ class SheetsChartElement(PageElementBase):
         )
         return [request]
 
-    def element_to_update_request(self, element_id: str) -> List[GSlidesAPIRequest]:
-        """Convert a SheetsChartElement to an update request for the Google Slides API."""
-        requests = self.alt_text_update_request(element_id)
-
-        if (
-            hasattr(self.sheetsChart, "sheetsChartProperties")
-            and self.sheetsChart.sheetsChartProperties is not None
-        ):
-            chart_properties = self.sheetsChart.sheetsChartProperties.to_api_format()
-            chart_request = UpdateSheetsChartPropertiesRequest(
-                objectId=element_id,
-                sheetsChartProperties=chart_properties,
-                fields=",".join(dict_to_dot_separated_field_list(chart_properties)),
-            )
-            requests.append(chart_request)
-
-        return requests
+    # def element_to_update_request(self, element_id: str) -> List[GSlidesAPIRequest]:
+    #     """Convert a SheetsChartElement to an update request for the Google Slides API."""
+    #     requests = self.alt_text_update_request(element_id)
+    #
+    #     if (
+    #         hasattr(self.sheetsChart, "sheetsChartProperties")
+    #         and self.sheetsChart.sheetsChartProperties is not None
+    #     ):
+    #         chart_properties = self.sheetsChart.sheetsChartProperties.to_api_format()
+    #         chart_request = UpdateSheetsChartPropertiesRequest(
+    #             objectId=element_id,
+    #             sheetsChartProperties=chart_properties,
+    #             fields=",".join(dict_to_dot_separated_field_list(chart_properties)),
+    #         )
+    #         requests.append(chart_request)
+    #
+    #     return requests
 
 
 class SpeakerSpotlightElement(PageElementBase):
@@ -363,7 +518,9 @@ class SpeakerSpotlightElement(PageElementBase):
 
     speakerSpotlight: SpeakerSpotlight
     type: ElementKind = Field(
-        default=ElementKind.SPEAKER_SPOTLIGHT, description="The type of page element", exclude=True
+        default=ElementKind.SPEAKER_SPOTLIGHT,
+        description="The type of page element",
+        exclude=True,
     )
 
     @field_validator("type")
@@ -402,7 +559,9 @@ class GroupElement(PageElementBase):
         """Convert a GroupElement to a create request for the Google Slides API."""
         # Note: Group creation is typically done by grouping existing elements
         # This is a placeholder implementation
-        raise NotImplementedError("Group creation should be done by grouping existing elements")
+        raise NotImplementedError(
+            "Group creation should be done by grouping existing elements"
+        )
 
     def element_to_update_request(self, element_id: str) -> List[GSlidesAPIRequest]:
         """Convert a GroupElement to an update request for the Google Slides API."""
@@ -425,3 +584,7 @@ PageElement = Annotated[
     ],
     Discriminator(element_discriminator),
 ]
+
+# Rebuild models to resolve forward references
+Group.model_rebuild()
+GroupElement.model_rebuild()
