@@ -1,21 +1,20 @@
-from typing import Any, Dict, List, Optional
+from typing import TYPE_CHECKING, Any, Dict, List, Optional
 
 from pydantic import Field
 
-from gslides_api.domain import (
-    BulletGlyphPreset,
-    GSlidesBaseModel,
-    LayoutReference,
-    ImageProperties,
-)
-from gslides_api import ShapeProperties
-from gslides_api.text import ShapeType, TextStyle
-from gslides_api.request.domain import (
-    Range,
-    TableCellLocation,
-    ElementProperties,
-    PlaceholderIdMapping,
-)
+from gslides_api.text import ShapeProperties
+from gslides_api.domain import (AffineTransform, BulletGlyphPreset,
+                                GSlidesBaseModel, ImageProperties,
+                                LayoutReference, LineProperties,
+                                PageElementProperties, VideoProperties)
+from gslides_api.request.domain import (PlaceholderIdMapping, Range,
+                                        SubstringMatchCriteria,
+                                        TableCellLocation)
+from gslides_api.text import ParagraphStyle, TextStyle, Type
+
+if TYPE_CHECKING:
+    from gslides_api.page.base import PageProperties
+    from gslides_api.page.slide_properties import SlideProperties
 
 
 class GSlidesAPIRequest(GSlidesBaseModel):
@@ -139,8 +138,10 @@ class CreateShapeRequest(GSlidesAPIRequest):
         default=None,
         description="A user-supplied object ID. If specified, the ID must be unique among all pages and page elements in the presentation.",
     )
-    elementProperties: ElementProperties = Field(description="The element properties for the shape")
-    shapeType: ShapeType = Field(description="The shape type")
+    elementProperties: PageElementProperties = Field(
+        description="The element properties for the shape"
+    )
+    shapeType: Type = Field(description="The shape type")
 
 
 class UpdateShapePropertiesRequest(GSlidesAPIRequest):
@@ -152,7 +153,9 @@ class UpdateShapePropertiesRequest(GSlidesAPIRequest):
     """
 
     objectId: str = Field(description="The object ID of the shape to update")
-    shapeProperties: ShapeProperties = Field(description="The shape properties to update")
+    shapeProperties: ShapeProperties = Field(
+        description="The shape properties to update"
+    )
     fields: str = Field(
         description="The fields that should be updated. At least one field must be specified. The root 'shapeProperties' is implied and should not be specified. A single '*' can be used as short-hand for listing every field."
     )
@@ -166,7 +169,9 @@ class ReplaceImageRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#replaceimagerequest
     """
 
-    imageObjectId: str = Field(description="The ID of the existing image that will be replaced")
+    imageObjectId: str = Field(
+        description="The ID of the existing image that will be replaced"
+    )
     url: str = Field(
         description="The image URL. The image is fetched once at insertion time and a copy is stored for display inside the presentation. Images must be less than 50MB in size, cannot exceed 25 megapixels, and must be in one of PNG, JPEG, or GIF format."
     )
@@ -212,7 +217,9 @@ class UpdateSlidePropertiesRequest(GSlidesAPIRequest):
     """
 
     objectId: str = Field(description="The object ID of the slide to update")
-    slideProperties: Dict[str, Any] = Field(description="The slide properties to update")
+    slideProperties: "SlideProperties" = Field(
+        description="The slide properties to update"
+    )
     fields: str = Field(
         description="The fields that should be updated. At least one field must be specified. The root 'slideProperties' is implied and should not be specified. A single '*' can be used as short-hand for listing every field."
     )
@@ -243,7 +250,9 @@ class UpdatePagePropertiesRequest(GSlidesAPIRequest):
     """
 
     objectId: str = Field(description="The object ID of the page to update")
-    pageProperties: Dict[str, Any] = Field(description="The page properties to update")
+    pageProperties: "PageProperties" = Field(
+        description="The page properties to update"
+    )
     fields: str = Field(
         description="The fields that should be updated. At least one field must be specified. The root 'pageProperties' is implied and should not be specified. A single '*' can be used as short-hand for listing every field."
     )
@@ -258,7 +267,9 @@ class DeleteObjectRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#deleteobjectrequest
     """
 
-    objectId: str = Field(description="The object ID of the page or page element to delete")
+    objectId: str = Field(
+        description="The object ID of the page or page element to delete"
+    )
 
 
 class DuplicateObjectRequest(GSlidesAPIRequest):
@@ -286,7 +297,9 @@ class UpdateImagePropertiesRequest(GSlidesAPIRequest):
     """
 
     objectId: str = Field(description="The object ID of the image to update")
-    imageProperties: ImageProperties = Field(description="The image properties to update")
+    imageProperties: ImageProperties = Field(
+        description="The image properties to update"
+    )
     fields: str = Field(
         description="The fields that should be updated. At least one field must be specified. The root 'imageProperties' is implied and should not be specified. A single '*' can be used as short-hand for listing every field."
     )
@@ -324,7 +337,9 @@ class UpdateVideoPropertiesRequest(GSlidesAPIRequest):
     """
 
     objectId: str = Field(description="The object ID of the video to update")
-    videoProperties: Dict[str, Any] = Field(description="The video properties to update")
+    videoProperties: VideoProperties = Field(
+        description="The video properties to update"
+    )
     fields: str = Field(
         description="The fields that should be updated. At least one field must be specified. The root 'videoProperties' is implied and should not be specified. A single '*' can be used as short-hand for listing every field."
     )
@@ -339,27 +354,29 @@ class UpdateLinePropertiesRequest(GSlidesAPIRequest):
     """
 
     objectId: str = Field(description="The object ID of the line to update")
-    lineProperties: Dict[str, Any] = Field(description="The line properties to update")
+    lineProperties: LineProperties = Field(description="The line properties to update")
     fields: str = Field(
         description="The fields that should be updated. At least one field must be specified. The root 'lineProperties' is implied and should not be specified. A single '*' can be used as short-hand for listing every field."
     )
 
 
-class UpdateSheetsChartPropertiesRequest(GSlidesAPIRequest):
-    """Updates the properties of a SheetsChart.
-
-    This request updates the sheets chart properties for the specified chart.
-
-    Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#updatesheetschartpropertiesrequest
-    """
-
-    objectId: str = Field(description="The object ID of the sheets chart to update")
-    sheetsChartProperties: Dict[str, Any] = Field(
-        description="The sheets chart properties to update"
-    )
-    fields: str = Field(
-        description="The fields that should be updated. At least one field must be specified. The root 'sheetsChartProperties' is implied and should not be specified. A single '*' can be used as short-hand for listing every field."
-    )
+#  This one appears to have been hallucinated, no trace of it in the actual docs
+# class UpdateSheetsChartPropertiesRequest(GSlidesAPIRequest):
+#     """Updates the properties of a SheetsChart.
+#
+#     This request updates the sheets chart properties for the specified chart.
+#
+#     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#updatesheetschartpropertiesrequest
+#     """
+#
+#     objectId: str = Field(description="The object ID of the sheets chart to update")
+#     sheetsChartProperties: Dict[str, Any] = Field(
+#         description="The sheets chart properties to update"
+#     )
+#     fields: str = Field(
+#         description="The fields that should be updated. At least one field must be specified. The root 'sheetsChartProperties' is implied and should not be specified. A single '*' can be used as short-hand for listing every field."
+#     )
+#
 
 
 class CreateImageRequest(GSlidesAPIRequest):
@@ -370,7 +387,13 @@ class CreateImageRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#createimagerequest
     """
 
-    elementProperties: Dict[str, Any] = Field(description="The element properties for the image")
+    objectId: Optional[str] = Field(
+        default=None,
+        description="A user-supplied object ID. If specified, the ID must be unique among all pages and page elements in the presentation.",
+    )
+    elementProperties: PageElementProperties = Field(
+        description="The element properties for the image"
+    )
     url: str = Field(description="The image URL")
 
 
@@ -382,7 +405,9 @@ class CreateVideoRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#createvideorequest
     """
 
-    elementProperties: Dict[str, Any] = Field(description="The element properties for the video")
+    elementProperties: PageElementProperties = Field(
+        description="The element properties for the video"
+    )
     source: str = Field(description="The video source type (e.g., 'YOUTUBE')")
     id: str = Field(description="The video ID")
 
@@ -395,20 +420,23 @@ class CreateLineRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#createlinerequest
     """
 
-    elementProperties: Dict[str, Any] = Field(description="The element properties for the line")
+    elementProperties: PageElementProperties = Field(
+        description="The element properties for the line"
+    )
     lineCategory: str = Field(description="The line category (e.g., 'STRAIGHT')")
 
 
-class CreateWordArtRequest(GSlidesAPIRequest):
-    """Creates a new word art.
-
-    This request creates a new word art on the specified page.
-
-    Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#createwordartrequest
-    """
-
-    elementProperties: Dict[str, Any] = Field(description="The element properties for the word art")
-    renderedText: str = Field(description="The text to render as word art")
+# This seems to have been hallucinated
+# class CreateWordArtRequest(GSlidesAPIRequest):
+#     """Creates a new word art.
+#
+#     This request creates a new word art on the specified page.
+#
+#     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#createwordartrequest
+#     """
+#
+#     elementProperties: Dict[str, Any] = Field(description="The element properties for the word art")
+#     renderedText: str = Field(description="The text to render as word art")
 
 
 class CreateSheetsChartRequest(GSlidesAPIRequest):
@@ -419,7 +447,7 @@ class CreateSheetsChartRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#createsheetschartrequest
     """
 
-    elementProperties: Dict[str, Any] = Field(
+    elementProperties: PageElementProperties = Field(
         description="The element properties for the sheets chart"
     )
     spreadsheetId: str = Field(
@@ -434,7 +462,7 @@ class ReplaceAllTextRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#replacealltextrequest
     """
 
-    containsText: Dict[str, Any] = Field(
+    containsText: SubstringMatchCriteria = Field(
         description="Finds all instances of text matching this substring"
     )
     replaceText: str = Field(description="The text that will replace the matched text")
@@ -451,7 +479,7 @@ class UpdatePageElementTransformRequest(GSlidesAPIRequest):
     """
 
     objectId: str = Field(description="The object ID of the page element to update")
-    transform: Dict[str, Any] = Field(
+    transform: AffineTransform = Field(
         description="The input transform matrix used to update the page element"
     )
     applyMode: str = Field(description="The apply mode of the transform update")
@@ -472,11 +500,13 @@ class ReplaceAllShapesWithImageRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#replaceallshapeswithimagerequest
     """
 
-    containsText: Dict[str, Any] = Field(
+    containsText: SubstringMatchCriteria = Field(
         description="If set, this request will replace all of the shapes that contain the given text"
     )
     imageUrl: Optional[str] = Field(default=None, description="The image URL")
-    imageReplaceMethod: Optional[str] = Field(default=None, description="The image replace method")
+    imageReplaceMethod: Optional[str] = Field(
+        default=None, description="The image replace method"
+    )
     pageObjectIds: Optional[List[str]] = Field(
         default=None,
         description="If non-empty, limits the matches to page elements only on the given pages",
@@ -489,7 +519,7 @@ class ReplaceAllShapesWithSheetsChartRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#replaceallshapeswithsheetschartrequest
     """
 
-    containsText: Dict[str, Any] = Field(
+    containsText: SubstringMatchCriteria = Field(
         description="The criteria that the shapes must match in order to be replaced"
     )
     spreadsheetId: str = Field(
@@ -517,11 +547,11 @@ class DeleteParagraphBulletsRequest(GSlidesAPIRequest):
     objectId: str = Field(
         description="The object ID of the shape or table containing the text to delete bullets from"
     )
-    cellLocation: Optional[Dict[str, Any]] = Field(
+    cellLocation: Optional[TableCellLocation] = Field(
         default=None,
         description="The optional table cell location if the text to be modified is in a table cell",
     )
-    textRange: Dict[str, Any] = Field(description="The range of text to delete bullets from")
+    textRange: Range = Field(description="The range of text to delete bullets from")
 
 
 class UpdateParagraphStyleRequest(GSlidesAPIRequest):
@@ -533,12 +563,12 @@ class UpdateParagraphStyleRequest(GSlidesAPIRequest):
     objectId: str = Field(
         description="The object ID of the shape or table with the text to be styled"
     )
-    cellLocation: Optional[Dict[str, Any]] = Field(
+    cellLocation: Optional[TableCellLocation] = Field(
         default=None,
         description="The location of the cell in the table containing the paragraph(s) to style",
     )
-    style: Dict[str, Any] = Field(description="The paragraph's style")
-    textRange: Dict[str, Any] = Field(
+    style: ParagraphStyle = Field(description="The paragraph's style")
+    textRange: Range = Field(
         description="The range of text containing the paragraph(s) to style"
     )
     fields: str = Field(description="The fields that should be updated")
@@ -551,9 +581,12 @@ class GroupObjectsRequest(GSlidesAPIRequest):
     """
 
     groupObjectId: Optional[str] = Field(
-        default=None, description="A user-supplied object ID for the group to be created"
+        default=None,
+        description="A user-supplied object ID for the group to be created",
+    )
+    childrenObjectIds: List[str] = Field(
+        description="The object IDs of the objects to group"
     )
-    childrenObjectIds: List[str] = Field(description="The object IDs of the objects to group")
 
 
 class UngroupObjectsRequest(GSlidesAPIRequest):
@@ -574,7 +607,9 @@ class UpdatePageElementsZOrderRequest(GSlidesAPIRequest):
     pageElementObjectIds: List[str] = Field(
         description="The object IDs of the page elements to update"
     )
-    operation: str = Field(description="The Z-order operation to apply on the page elements")
+    operation: str = Field(
+        description="The Z-order operation to apply on the page elements"
+    )
 
 
 class UpdateLineCategoryRequest(GSlidesAPIRequest):
@@ -583,7 +618,9 @@ class UpdateLineCategoryRequest(GSlidesAPIRequest):
     Reference: https://developers.google.com/workspace/slides/api/reference/rest/v1/presentations/request#updatelinecategoryrequest
     """
 
-    objectId: str = Field(description="The object ID of the line the update is applied to")
+    objectId: str = Field(
+        description="The object ID of the line the update is applied to"
+    )
     lineCategory: str = Field(description="The line category to update to")
 
 
